@@ -95,7 +95,7 @@ EOF
 		# IKIWIKI_CGILOCK_FD so unlockwiki can close it.
 		$pre_exec=<<"EOF";
 	lockfd=open("$config{wikistatedir}/cgilock", O_CREAT | O_RDWR, 0666);
-	if (lockfd != -1 && flock(lockfd, LOCK_EX) == 0) {
+	if (lockfd != -1 && lockf(lockfd, F_LOCK, 0) == 0) {
 		char *fd_s=malloc(8);
 		sprintf(fd_s, "%i", lockfd);
 		setenv("IKIWIKI_CGILOCK_FD", fd_s, 1);
@@ -214,7 +214,7 @@ $set_background_command
 EOF
 
 	my @cc=exists $ENV{CC} ? possibly_foolish_untaint($ENV{CC}) : 'cc';
-	push @cc, possibly_foolish_untaint($ENV{CFLAGS}) if exists $ENV{CFLAGS};
+	push @cc, split(' ', possibly_foolish_untaint($ENV{CFLAGS})) if exists $ENV{CFLAGS};
 	if (system(@cc, "$wrapper.c", "-o", "$wrapper.new") != 0) {
 		#translators: The parameter is a C filename.
 		error(sprintf(gettext("failed to compile %s"), "$wrapper.c"));
